<?php
/**
 * @version    CVS: 1.0.0
 * @package    Com_Monetka
 * @author     MIKHAIL KUCHEROV <Michael@xstartx.ru>
 * @copyright  2024 MIKHAIL KUCHEROV
 * @license    GNU General Public License версии 2 или более поздней; Смотрите LICENSE.txt
 */

namespace Xstartx\Component\Monetka\Administrator\Controller;

\defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\MVC\Controller\AdminController;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\Log\Log;
use Xstartx\Component\Monetka\Administrator\Helper\MonetkaHelper;

/**
* Контроллер для списка балансов Monetka.
*
* @since  1.0.0
*/
class Monetka_balancesController extends AdminController
{
    private $jomsocialEnabled;
    private $fieldsExist;

    public function __construct($config = [])
    {
        parent::__construct($config);
        $this->jomsocialEnabled = MonetkaHelper::isJomsocialEnabled();
        $this->fieldsExist = MonetkaHelper::checkFieldsExist();
    }

    /**
     * Проксирование для getModel.
     *
     * @param   string  $name    Имя модели
     * @param   string  $prefix  Префикс класса
     * @param   array   $config  Конфигурация модели
     *
     * @return  object
     */
    public function getModel($name = 'Monetka_balances', $prefix = 'Administrator', $config = array())
    {
        return parent::getModel($name, $prefix, $config);
    }

    /**
     * Метод для обновления баланса пользователя.
     *
     * @return void
     */
    public function updateBalance()
    {
        // Проверка токена безопасности
        $this->checkToken();

        // Получаем входные данные
        $userId = $this->input->post->getInt('user_id'); // ID пользователя Joomla
        $balance = $this->input->post->get('balance', null, 'raw'); // Получаем как строку для последующего преобразования
        $groupId = $this->input->post->getInt('group_id', null); // Group ID, если есть

        // Логирование полученных данных
        Log::add('Получены данные для обновления баланса: user_id=' . $userId . ', balance=' . $balance . ', group_id=' . ($groupId !== null ? $groupId : 'null'), Log::DEBUG, 'monetka');

        // Преобразование баланса в число с плавающей точкой
        $balance = (float) $balance;

        // Проверяем наличие данных
        if (!$userId || is_nan($balance)) { // Проверяем на NaN, если баланс не может быть преобразован в число
            Log::add('Некорректные входные данные для обновления баланса.', Log::ERROR, 'monetka');
            Factory::getApplication()->enqueueMessage(Text::_('COM_MONETKA_INVALID_INPUT'), 'error');
            $this->setRedirect('index.php?option=com_monetka&view=monetka_balances');
            return;
        }

        try {
            if ($this->fieldsExist) {
                // Если поля есть, обновляем с учетом группы
                if (!MonetkaHelper::updateUserBalance($userId, $balance, $groupId)) {
                    throw new \RuntimeException(Text::_('COM_MONETKA_BALANCE_UPDATE_ERROR'));
                }
            } else {
                // Если полей нет, обновляем просто по user_id
                if (!MonetkaHelper::updateUserBalance($userId, $balance)) {
                    throw new \RuntimeException(Text::_('COM_MONETKA_BALANCE_UPDATE_ERROR'));
                }
            }
            
            Log::add('Баланс успешно обновлен для пользователя ' . $userId . ', новый баланс: ' . $balance, Log::INFO, 'monetka');
            Factory::getApplication()->enqueueMessage(Text::_('COM_MONETKA_BALANCE_UPDATED_SUCCESS'), 'message');
        } catch (\Exception $e) {
            // Обработка ошибок
            Log::add('Ошибка при обновлении баланса: ' . $e->getMessage(), Log::ERROR, 'monetka');
            Factory::getApplication()->enqueueMessage($e->getMessage(), 'error');
        }

        // Перенаправление на список балансов
        $this->setRedirect('index.php?option=com_monetka&view=monetka_balances');
    }

    /**
     * Метод для сохранения балансов пользователей.
     *
     * @return void
     */
    public function saveBalances()
    {
        $this->checkToken();

        $input = Factory::getApplication()->input;
        $balances = $input->post->get('balances', [], 'array');
        if (empty($balances)) {
            Log::add('Массив балансов пуст.', Log::ERROR, 'monetka');
            Factory::getApplication()->enqueueMessage(Text::_('COM_MONETKA_SAVE_FAILED') . ': Empty balances array.', 'error');
            $this->setRedirect('index.php?option=com_monetka&view=monetka_balances');
            return;
        }

        try {
            Log::add('Начало сохранения балансов.', Log::INFO, 'monetka');

            if ($this->fieldsExist) {
                // Обработка балансов с учетом групп
                foreach ($balances as $userId => $groupBalances) {
                    foreach ($groupBalances as $groupId => $balance) {
                        if (!is_numeric($userId) || !is_numeric($groupId) || !is_numeric($balance)) {
                            throw new \InvalidArgumentException("Invalid data in balances array for user $userId, group $groupId.");
                        }

                        if (!MonetkaHelper::updateUserBalance($userId, (float) $balance, $groupId)) {
                            throw new \RuntimeException(Text::_('COM_MONETKA_SAVE_FAILED'));
                        }
                    }
                }
            } else {
                // Обработка балансов без групп
                foreach ($balances as $userId => $balance) {
                    if (!is_numeric($userId) || !is_numeric($balance)) {
                        throw new \InvalidArgumentException("Invalid data in balances array for user $userId.");
                    }

                    if (!MonetkaHelper::updateUserBalance($userId, (float) $balance)) {
                        throw new \RuntimeException(Text::_('COM_MONETKA_SAVE_FAILED'));
                    }
                }
            }

            Log::add('Все балансы успешно сохранены.', Log::INFO, 'monetka');
            Factory::getApplication()->enqueueMessage(Text::_('COM_MONETKA_BALANCES_SAVED_SUCCESS'), 'message');
        } catch (\Exception $e) {
            Log::add('Ошибка при сохранении балансов: ' . $e->getMessage(), Log::ERROR, 'monetka');
            Factory::getApplication()->enqueueMessage($e->getMessage(), 'error');
        }

        $this->setRedirect('index.php?option=com_monetka&view=monetka_balances');
    }

    public function cancelChanges()
    {
        $this->checkToken();
        // Логика для отмены изменений
        Log::add('Изменения балансов отменены.', Log::INFO, 'monetka');
        $this->setMessage(Text::_('COM_MONETKA_BALANCES_CHANGES_CANCELED'));
        $this->setRedirect('index.php?option=com_monetka&view=monetka_balances');
    }
    
    public function saveAndClose()
    {
        $this->saveBalances();
        Log::add('Балансы сохранены и закрыто окно.', Log::INFO, 'monetka');
        $this->setRedirect('index.php?option=com_config');
    }
}
 
