<?php
/**
 * @version    CVS: 1.0.0
 * @package    Com_Monetka
 * @author     MIKHAIL KUCHEROV <Michael@xstartx.ru>
 * @copyright  2024 MIKHAIL KUCHEROV
 * @license    GNU General Public License версии 2 или более поздней; Смотрите LICENSE.txt
 */

namespace Xstartx\Component\Monetka\Administrator\Helper;

use Joomla\CMS\Factory;
use Joomla\CMS\Helper\ContentHelper;
use Joomla\Database\ParameterType;
use Joomla\CMS\Plugin\PluginHelper;

// No direct access
defined('_JEXEC') or die;

/**
 * Monetka helper.
 *
 * @since  1.0.0
 */
class MonetkaHelper extends ContentHelper
{
    /**
     * Проверяет, существуют ли поля group_id и group_name в таблице базы данных.
     *
     * @return bool True, если поля существуют.
     */
    public static function checkFieldsExist()
    {
        $db = Factory::getContainer()->get('DatabaseDriver');
        $columns = $db->getTableColumns('#__monetka_users', false);

        return isset($columns['group_id'], $columns['group_name']);
    }

    /**
     * Проверяет, активен ли плагин Jomsocial.
     *
     * @return bool True, если плагин активен.
     */
    public static function isJomsocialEnabled()
    {
        return (bool) PluginHelper::getPlugin('monetka', 'jomsocial');
    }

    /**
     * Получить баланс пользователя
     *
     * @param   int  $userId  Идентификатор пользователя
     * @param   int|null  $groupId  Идентификатор группы, если применимо
     *
     * @return  float  Баланс пользователя
     */
    public static function getUserBalance($userId, $groupId = null)
    {
        $db = Factory::getContainer()->get('DatabaseDriver');
        $fieldsExist = self::checkFieldsExist();
        $jomsocialEnabled = self::isJomsocialEnabled();

        $query = $db->getQuery(true)
                    ->select('balance')
                    ->from($db->quoteName('#__monetka_users'))
                    ->where($db->quoteName('user_id') . ' = :userId')
                    ->bind(':userId', $userId, ParameterType::INTEGER);

        if ($fieldsExist) {
            if ($groupId !== null) {
                $query->where($db->quoteName('group_id') . ' = :groupId')
                      ->bind(':groupId', $groupId, ParameterType::INTEGER);
            } else {
                // Если groupId не указан, выбираем общий баланс (group_id = 0)
                $query->where($db->quoteName('group_id') . ' = 0');
            }
        }

        $db->setQuery($query);

        return (float) $db->loadResult();
    }

    /**
     * Обновить баланс пользователя
     *
     * @param   int     $userId  Идентификатор пользователя
     * @param   float   $amount  Сумма для обновления баланса
     * @param   int|null $groupId Идентификатор группы, если применимо
     *
     * @return  bool    Статус операции
     */
    public static function updateUserBalance($userId, $amount, $groupId = null)
    {
        $db = Factory::getContainer()->get('DatabaseDriver');
        $fieldsExist = self::checkFieldsExist();

        $query = $db->getQuery(true)
                    ->update($db->quoteName('#__monetka_users'))
                    ->set($db->quoteName('balance') . ' = :amount')
                    ->where($db->quoteName('user_id') . ' = :userId')
                    ->bind(':amount', $amount, ParameterType::STRING)
                    ->bind(':userId', $userId, ParameterType::INTEGER);

        if ($fieldsExist) {
            if ($groupId !== null) {
                $query->where($db->quoteName('group_id') . ' = :groupId')
                      ->bind(':groupId', $groupId, ParameterType::INTEGER);
            } else {
                // Если groupId не указан, обновляем общий баланс (group_id = 0)
                $query->where($db->quoteName('group_id') . ' = 0');
            }
        }

        $db->setQuery($query);

        return $db->execute() !== false;
    }

    /**
     * Получить список балансов пользователей, учитывая наличие плагина и полей в БД
     *
     * @return array Массив балансов
     */
    public static function getBalancesList()
    {
        $db = Factory::getContainer()->get('DatabaseDriver');
        $fieldsExist = self::checkFieldsExist();
        $jomsocialEnabled = self::isJomsocialEnabled();

        $query = $db->getQuery(true)
                    ->select(['user_id', 'username', 'balance', 'group_id', 'group_name'])
                    ->from($db->quoteName('#__monetka_users'));

        if ($fieldsExist && !$jomsocialEnabled) {
            // Если поля есть, но плагин не активен, удаляем строки с балансами по группам
            $query->where($db->quoteName('group_id') . ' = 0');
        }

        $db->setQuery($query);

        $results = $db->loadObjectList();

        return $results;
    }
}
 
