<?php
/**
 * @version    CVS: 1.0.0
 * @package    Com_Monetka
 */

namespace Xstartx\Component\Monetka\Administrator\Model;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\Language\Text;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Form\Form;
use Xstartx\Component\Monetka\Administrator\Helper\MonetkaHelper;
use Joomla\CMS\Pagination\Pagination;

defined('_JEXEC') or die;

/**
 * Модель для управления списком балансов Monetka.
 */
class Monetka_balancesModel extends ListModel
{
    private $fieldsExist;
    private $jomsocialEnabled;

    /**
     * Конструктор модели.
     *
     * @param array $config Настройки модели.
     * @param MVCFactoryInterface|null $factory MVC фабрика.
     */
    public function __construct($config = [], ?MVCFactoryInterface $factory = null)
    {
        $factory = $factory ?: Factory::getContainer()->get(MVCFactoryInterface::class);

        if (empty($config['filter_fields'])) {
            $config['filter_fields'] = [
                'id', 'a.id',
                'user_id', 'a.user_id',
                'username', 'a.username',
                'balance', 'a.balance',
                'group_id', 'a.group_id',
                'group_name', 'a.group_name',
            ];
        }

        parent::__construct($config, $factory);

        // Проверяем наличие полей в таблице и активность плагина Jomsocial
        $this->fieldsExist = MonetkaHelper::checkFieldsExist();
        $this->jomsocialEnabled = MonetkaHelper::isJomsocialEnabled();
    }

    /**
     * Получает элементы из базы данных.
     *
     * @return array Массив элементов.
     */
    public function getItems()
    {
        $items = parent::getItems();
    
        // Если записи отсутствуют, возвращаем пустой массив
        if (empty($items)) {
            Factory::getApplication()->enqueueMessage(Text::_('COM_MONETKA_NO_ITEMS_FOUND'), 'warning');
            return [];
        }
    
        if ($this->fieldsExist && $this->jomsocialEnabled) {
            $items = $this->addManagerInfo($items);
        }
    
        // Если плагин отключен, но поля существуют, убираем строки с группами
        if (!$this->jomsocialEnabled && $this->fieldsExist) {
            $items = array_filter($items, function($item) {
                return $item->group_id == 0;
            });
        }
    
        return array_values($items);
    }

    /**
     * Добавляет информацию о менеджере группы к элементам.
     *
     * @param array $items Массив элементов для обновления.
     * @return array Обновленный массив элементов.
     */
    private function addManagerInfo($items)
    {
        foreach ($items as &$item) {
            // Проверяем, что это не общий баланс (где group_id равно 0)
            if ($item->group_id != 0 && $item->user_id == $item->ownerid) {
                $item->manager = 'Менеджер';
            } else {
                $item->manager = '';
            }
        }
        return $items;
    }

    /**
     * Получает объект пагинации.
     *
     * @return JPagination Объект пагинации.
     */
    public function getPagination()
    {
        if (empty($this->pagination)) {
            $this->pagination = new Pagination(
                $this->getTotal(), 
                $this->getState('list.start'), 
                $this->getState('list.limit')
            );
        }
        return $this->pagination;
    }

    public function getTotal()
    {
        if (empty($this->total)) {
            $query = $this->getListQuery();
            $this->total = $this->_getListCount($query);
        }
        return $this->total;
    }
    
    protected function _getListCount($query)
    {
        $db = Factory::getContainer()->get('DatabaseDriver');
        $db->setQuery($query);
        $db->execute();
        return (int) $db->getNumRows();
    }

    /**
     * Получает форму фильтра.
     *
     * @param array $data Данные для формы.
     * @param bool $loadData Загружать ли данные.
     * @return JForm Объект формы.
     */
    public function getFilterForm($data = [], $loadData = true)
    {
        $form = parent::getFilterForm($data, $loadData);

        if (!$this->fieldsExist || !$this->jomsocialEnabled) {
            // Убираем поля, связанные с группами, если они не нужны
            $form->removeField('group_id');
            $form->removeField('group_name');
        }

        return $form;
    }

    /**
     * Устанавливает состояние модели.
     *
     * @param string $ordering Поле для сортировки.
     * @param string $direction Направление сортировки.
     */
    protected function populateState($ordering = 'a.id', $direction = 'ASC')
    {
        $app = Factory::getApplication();
    
        // Включаем пагинацию
        $limit = $app->getInput()->get('limit', $app->get('list_limit', 20), 'uint');
        $this->setState('list.limit', $limit);
    
        $limitstart = $app->getInput()->getUInt('limitstart', 0);
        $this->setState('list.start', $limitstart);
    
        // Устанавливаем состояние для поиска
        $search = $app->getInput()->getString('filter_search');
        $this->setState('filter.search', $search);
    
        // Вызываем родительский метод для установки сортировки или других параметров состояния
        parent::populateState($ordering, $direction);
    }

    /**
     * Формирует запрос для получения списка балансов.
     *
     * @return JDatabaseQuery Запрос базы данных.
     */
    protected function getListQuery()
    {
        $db = Factory::getContainer()->get('DatabaseDriver'); 
        $query = $db->getQuery(true); 

        if ($this->fieldsExist) {
            $query->select([
                'a.id',
                'a.user_id',
                'a.username',
                'a.balance',
                'a.group_id',
                'COALESCE(a.group_name, ' . $db->quote('Общий баланс') . ') AS group_name',
                'a.ownerid' // Добавляем ownerid в список выбираемых полей
            ])
            ->from($db->quoteName('#__monetka_users', 'a'));

            if (!$this->jomsocialEnabled) {
                $query->where($db->quoteName('group_id') . ' = 0');
            }
        } else {
            $query->select([
                'a.id',
                'a.user_id',
                'a.username',
                'a.balance'
            ])
            ->from($db->quoteName('#__monetka_users', 'a'));
        }

        // Фильтр поиска
        $search = $this->getState('filter.search');
        if (!empty($search)) {
            $query->where($db->quoteName('a.username') . ' LIKE ' . $db->quote('%' . $search . '%'));
        }

        // Добавляем сортировку
        $orderCol = $this->state->get('list.ordering', 'a.id');
        $orderDirn = $this->state->get('list.direction', 'ASC');
        $query->order($db->escape($orderCol . ' ' . $orderDirn));

        return $query;
    }

    /**
     * Обновляет баланс пользователя в таблице.
     *
     * @param int   $userId     ID пользователя в таблице монеток.
     * @param float $newBalance Новый баланс.
     * @param int|null $groupId ID группы (если применимо).
     * @return bool Возвращает true, если баланс успешно обновлён, иначе false.
     */
    public function updateBalance($userId, $newBalance, $groupId = null)
    {
        return MonetkaHelper::updateUserBalance($userId, $newBalance, $groupId);
    }

    /**
     * Сохраняет массив балансов.
     *
     * @param array $balances Ассоциативный массив [user_id => balance].
     * @return void
     * @throws RuntimeException
     */
    public function saveBalances(array $balances)
    {
        $db = Factory::getContainer()->get('DatabaseDriver');
    
        $db->transactionStart();
    
        try {
            foreach ($balances as $userId => $balanceData) {
                if (is_array($balanceData)) {
                    foreach ($balanceData as $groupId => $balance) {
                        if (!$this->updateBalance($userId, $balance, $groupId)) {
                            throw new \RuntimeException(Text::sprintf('COM_MONETKA_SAVE_FAILED_FOR_USER', $userId));
                        }
                    }
                } else {
                    if (!$this->updateBalance($userId, $balanceData)) {
                        throw new \RuntimeException(Text::sprintf('COM_MONETKA_SAVE_FAILED_FOR_USER', $userId));
                    }
                }
            }
    
            $db->transactionCommit();
            Factory::getApplication()->enqueueMessage(Text::_('COM_MONETKA_BALANCES_SAVED_SUCCESS'), 'message');
        } catch (\Exception $e) {
            $db->transactionRollback();
            Factory::getApplication()->enqueueMessage(Text::_('COM_MONETKA_SAVE_FAILED') . ': ' . $e->getMessage(), 'error');
            throw $e;
        }
    }

    /**
     * Получает группу пользователя.
     *
     * @param int $userId ID пользователя.
     * @return string|null Название группы или null, если возникла ошибка.
     */
    public function getUserGroup($userId)
    {
        $db = Factory::getContainer()->get('DatabaseDriver');
        $query = $db->getQuery(true);

        $query->select('g.title AS group_name')
            ->from($db->quoteName('#__user_usergroup_map', 'm'))
            ->join('INNER', $db->quoteName('#__usergroups', 'g') . ' ON g.id = m.group_id')
            ->where('m.user_id = ' . (int) $userId);

        $db->setQuery($query);

        try {
            return $db->loadResult();
        } catch (\RuntimeException $e) {
            Factory::getApplication()->enqueueMessage(Text::_('COM_MONETKA_ERROR_LOADING_GROUP'), 'error');
            return null;
        }
    }

    public function getForm($data = [], $loadData = true)
    {
        // Загружаем форму для балансов
        Form::addFormPath(realpath(JPATH_ROOT . '/administrator/components/com_monetka/models/forms'));
        $form = $this->loadForm('com_monetka.monetka_balances', 'monetka_balances', ['control' => 'jform', 'load_data' => $loadData]);
    
        if (!$form) {
            return false;
        }
    
        // Проверка на наличие и активность плагина Jomsocial
        if (!$this->jomsocialEnabled) {
            // Если плагин не активен, удаляем поля group_id и group_name из формы
            $form->removeField('group_id');
            $form->removeField('group_name');
        }

        return $form;
    }
}
 
