<?php

namespace Xstartx\Component\Monetka\Administrator\Model;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Model\AdminModel;
use Joomla\CMS\Language\Text;

defined('_JEXEC') or die;

/**
 * Модель для управления настройками.
 */
class SettingsModel extends AdminModel
{
    /**
     * Получение формы для редактирования данных.
     *
     * @param array $data Данные для формы.
     * @param bool  $loadData Загружать данные.
     * @return \Joomla\CMS\Form\Form|null Форма или null в случае ошибки.
     */
    public function getForm($data = [], $loadData = true)
    {
        try {
            $form = $this->loadForm(
                'com_monetka.settings',
                'settings',
                ['control' => 'jform', 'load_data' => $loadData]
            );

            if (!$form) {
                throw new \RuntimeException(Text::_('COM_MONETKA_ERROR_FORM_NOT_FOUND'));
            }

            return $form;
        } catch (\Exception $e) {
            Factory::getApplication()->enqueueMessage($e->getMessage(), 'error');
            return null;
        }
    }

    /**
     * Загрузка данных для формы.
     *
     * @return array Данные формы.
     */
    public function loadFormData()
    {
        try {
            $data = $this->getSettings();

            if (empty($data)) {
                // Если запись отсутствует, создаём её
                $this->createDefaultSettings();
                $data = $this->getSettings();
            }

            return (array) $data ?: [];
        } catch (\Exception $e) {
            Factory::getApplication()->enqueueMessage(Text::_('COM_MONETKA_ERROR_LOADING_FORM_DATA') . ': ' . $e->getMessage(), 'error');
            return [];
        }
    }

    /**
     * Сохранение данных настроек.
     *
     * @param array $data Данные для сохранения.
     * @return bool Результат сохранения.
     */
    public function saveSettings($data)
    {
        try {
            // Убедитесь, что 'use_kucoin' включено в данные
            if (!isset($data['use_kucoin'])) {
                $data['use_kucoin'] = 0; // По умолчанию, если не указано
            }
            
            $table = $this->getTable();

            // Загружаем текущие данные
            if (!$table->load(1)) {
                Factory::getApplication()->enqueueMessage(Text::_('COM_MONETKA_ERROR_LOADING_SETTINGS'), 'error');
                return false;
            }

            // Привязываем новые данные
            $table->bind($data);

            // Проверяем данные перед сохранением
            if (!$table->check()) {
                return false; // Ошибка уже добавлена в очередь сообщений
            }

            // Сохраняем данные
            if (!$table->store()) {
                Factory::getApplication()->enqueueMessage(Text::_('COM_MONETKA_ERROR_SAVING_SETTINGS'), 'error');
                return false;
            }

            return true;
        } catch (\Exception $e) {
            Factory::getApplication()->enqueueMessage(Text::_('COM_MONETKA_ERROR_SAVING_SETTINGS') . ': ' . $e->getMessage(), 'error');
            return false;
        }
    }

    /**
     * Получение объекта таблицы.
     *
     * @param string $type   Имя таблицы.
     * @param string $prefix Префикс таблицы.
     * @param array  $config Конфигурация.
     * @return \Joomla\CMS\Table\Table Объект таблицы.
     */
    public function getTable($type = 'Settings', $prefix = 'Xstartx\\Component\\Monetka\\Administrator\\Table', $config = [])
    {
        return parent::getTable($type, $prefix, $config);
    }

    /**
     * Получение настроек из базы данных.
     *
     * @return object|null Настройки или null в случае ошибки.
     */
    public function getSettings()
    {
        try {
            $db = Factory::getContainer()->get('DatabaseDriver');
            $query = $db->getQuery(true)
                ->select('*')
                ->from($db->quoteName('#__monetka_settings'))
                ->where($db->quoteName('id') . ' = 1');

            $db->setQuery($query);
            return $db->loadObject();
        } catch (\Exception $e) {
            Factory::getApplication()->enqueueMessage(Text::_('COM_MONETKA_ERROR_LOADING_SETTINGS') . ': ' . $e->getMessage(), 'error');
            return null;
        }
    }

    /**
     * Создание записи по умолчанию в таблице настроек.
     *
     * @return bool Успешность создания.
     */
    public function createDefaultSettings()
    {
        try {
            $db = Factory::getContainer()->get('DatabaseDriver');
            $query = $db->getQuery(true);

            $query->insert($db->quoteName('#__monetka_settings'))
                ->columns([
                    'id', 'site_coin_name', 'coins_per_group', 'site_fee_coins', 
                    'use_kucoin', 'kucoin_api_key', 'kucoin_api_secret', 'kucoin_api_passphrase'
                ])
                ->values('1, "Монетка", 0, 0.0, 0, "", "", ""');

            $db->setQuery($query);
            $db->execute();
            return true;
        } catch (\Exception $e) {
            Factory::getApplication()->enqueueMessage(Text::_('COM_MONETKA_ERROR_CREATING_DEFAULT_SETTINGS') . ': ' . $e->getMessage(), 'error');
            return false;
        }
    }
}
 
